@@ -24,6 +24,7 @@
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdint.h>
+#include <stdio.h>
 
 #include "lv2/lv2plug.in/ns/lv2core/lv2.h"
 
@@ -52,6 +53,7 @@ extern "C" {
 #define LILV_NS_FOAF "http://xmlns.com/foaf/0.1/"
 #define LILV_NS_LILV "http://drobilla.net/ns/lilv#"
 #define LILV_NS_LV2  "http://lv2plug.in/ns/lv2core#"
+#define LILV_NS_OWL  "http://www.w3.org/2002/07/owl#"
 #define LILV_NS_RDF  "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 #define LILV_NS_RDFS "http://www.w3.org/2000/01/rdf-schema#"
 #define LILV_NS_XSD  "http://www.w3.org/2001/XMLSchema#"
@@ -899,6 +901,27 @@ LILV_API
 bool
 lilv_plugin_is_replaced(const LilvPlugin* plugin);
 
+/**
+   Write the Turtle description of @c plugin to @c file.
+
+   This function is particularly useful for porting plugins in conjunction with
+   an LV2 bridge such as NASPRO.
+*/
+LILV_API
+void
+lilv_plugin_write_description(LilvWorld*        world,
+                              const LilvPlugin* plugin,
+                              const LilvNode*   base_uri,
+                              FILE*             plugin_file);
+
+LILV_API
+void
+lilv_plugin_write_manifest_entry(LilvWorld*        world,
+                                 const LilvPlugin* plugin,
+                                 const LilvNode*   base_uri,
+                                 FILE*             manifest_file,
+                                 const char*       plugin_file_path);
+
 /**
    @}
    @name Port
@@ -977,7 +1000,7 @@ lilv_port_get_classes(const LilvPlugin* plugin,
    Determine if a port is of a given class (input, output, audio, etc).
    For convenience/performance/extensibility reasons, hosts are expected to
    create a LilvNode for each port class they "care about".  Well-known type
-   URI strings are defined (e.g. LILV_PORT_CLASS_INPUT) for convenience, but
+   URI strings are defined (e.g. LILV_URI_INPUT_PORT) for convenience, but
    this function is designed so that Lilv is usable with any port types
    without requiring explicit support in Lilv.
 */
@@ -989,7 +1012,7 @@ lilv_port_is_a(const LilvPlugin* plugin,
 
 /**
    Get the default, minimum, and maximum values of a port.
-   
+
    @a def, @a min, and @a max are outputs, pass pointers to uninitialized
    LilvNode* variables.  These will be set to point at new values (which must
    be freed by the caller using lilv_node_free), or NULL if the value does not
