@@ -72,40 +72,40 @@ lilv_plugin_instantiate(const LilvPlugin*        plugin,
 				            lib_path);
 				dlclose(lib);
 				break;  // return NULL
+			}
+
+			// Parse bundle URI to use as base URI
+			const LilvNode* bundle_uri     = lilv_plugin_get_bundle_uri(plugin);
+			const char*     bundle_uri_str = lilv_node_as_uri(bundle_uri);
+			SerdURI         base_uri;
+			if (serd_uri_parse((const uint8_t*)bundle_uri_str, &base_uri)) {
+				dlclose(lib);
+				break;
+			}
+
+			// Resolve library plugin URI against base URI
+			SerdURI  abs_uri;
+			SerdNode abs_uri_node = serd_node_new_uri_from_string(
+				(const uint8_t*)ld->URI, &base_uri, &abs_uri);
+			if (!abs_uri_node.buf) {
+				LILV_ERRORF("Failed to parse plugin URI `%s'\n", ld->URI);
+				dlclose(lib);
+				break;
+			}
+
+			if (!strcmp((const char*)abs_uri_node.buf,
+			            lilv_node_as_uri(lilv_plugin_get_uri(plugin)))) {
+				// Create LilvInstance to return
+				result = malloc(sizeof(struct LilvInstanceImpl));
+				result->lv2_descriptor = ld;
+				result->lv2_handle = ld->instantiate(
+					ld, sample_rate, (char*)bundle_path,
+					(features) ? features : local_features);
+				result->pimpl = lib;
+				serd_node_free(&abs_uri_node);
+				break;
 			} else {
-				// Parse bundle URI to use as base URI
-				const LilvNode* bundle_uri     = lilv_plugin_get_bundle_uri(plugin);
-				const char*     bundle_uri_str = lilv_node_as_uri(bundle_uri);
-				SerdURI         base_uri;
-				if (serd_uri_parse((const uint8_t*)bundle_uri_str, &base_uri)) {
-					dlclose(lib);
-					break;
-				}
-
-				// Resolve library plugin URI against base URI
-				SerdURI  abs_uri;
-				SerdNode abs_uri_node = serd_node_new_uri_from_string(
-					(const uint8_t*)ld->URI, &base_uri, &abs_uri);
-				if (!abs_uri_node.buf) {
-					LILV_ERRORF("Failed to parse plugin URI `%s'\n", ld->URI);
-					dlclose(lib);
-					break;
-				}
-
-				if (!strcmp((const char*)abs_uri_node.buf,
-				            lilv_node_as_uri(lilv_plugin_get_uri(plugin)))) {
-					// Create LilvInstance to return
-					result = malloc(sizeof(struct LilvInstanceImpl));
-					result->lv2_descriptor = ld;
-					result->lv2_handle = ld->instantiate(
-						ld, sample_rate, (char*)bundle_path,
-						(features) ? features : local_features);
-					result->pimpl = lib;
-					serd_node_free(&abs_uri_node);
-					break;
-				} else {
-					serd_node_free(&abs_uri_node);
-				}
+				serd_node_free(&abs_uri_node);
 			}
 		}
 	}
