@@ -18,6 +18,8 @@
 
 #include <assert.h>
 #include <math.h>
+#include <stdarg.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -157,6 +159,8 @@ lilv_plugin_load(LilvPlugin* p)
 		GetDataFunc get_data_func = (GetDataFunc)lilv_dlfunc(
 			lib, "lv2_dyn_manifest_get_data");
 		if (get_data_func) {
+			serd_env_set_base_uri(
+				env, sord_node_to_serd_node(p->dynman_uri->val.uri_val));
 			FILE* fd = tmpfile();
 			get_data_func(handle, fd, lilv_node_as_string(p->plugin_uri));
 			rewind(fd);
@@ -241,9 +245,9 @@ lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_p)
 			FOREACH_MATCH(types) {
 				const SordNode* type = lilv_match_object(types);
 				if (sord_node_get_type(type) == SORD_URI) {
-					lilv_array_append(
+					zix_tree_insert(
 						this_port->classes,
-						lilv_node_new_from_node(p->world, type));
+						lilv_node_new_from_node(p->world, type), NULL);
 				} else {
 					LILV_WARNF("Plugin <%s> port type is not a URI\n",
 					           lilv_node_as_uri(p->plugin_uri));
@@ -457,20 +461,35 @@ lilv_plugin_get_port_ranges_float(const LilvPlugin* p,
 	for (uint32_t i = 0; i < p->num_ports; ++i) {
 		lilv_port_get_range(p, p->ports[i], defptr, minptr, maxptr);
 
-		if (min_values)
-			min_values[i] = lilv_node_is_float(min)
-				? lilv_node_as_float(min)
-				: NAN;
+		if (min_values) {
+			if (lilv_node_is_float(min)) {
+				min_values[i] = lilv_node_as_float(min);
+			} else if (lilv_node_is_int(min)) {
+				min_values[i] = lilv_node_as_int(min);
+			} else {
+				min_values[i] = NAN;
+			}
+		}
 
-		if (max_values)
-			max_values[i] = lilv_node_is_float(max)
-				? lilv_node_as_float(max)
-				: NAN;
+		if (max_values) {
+			if (lilv_node_is_float(max)) {
+				max_values[i] = lilv_node_as_float(max);
+			} else if (lilv_node_is_int(max)) {
+				max_values[i] = lilv_node_as_int(max);
+			} else {
+				max_values[i] = NAN;
+			}
+		}
 
-		if (def_values)
-			def_values[i] = lilv_node_is_float(def)
-				? lilv_node_as_float(def)
-				: NAN;
+		if (def_values) {
+			if (lilv_node_is_float(def)) {
+				def_values[i] = lilv_node_as_float(def);
+			} else if (lilv_node_is_int(def)) {
+				def_values[i] = lilv_node_as_int(def);
+			} else {
+				def_values[i] = NAN;
+			}
+		}
 
 		lilv_node_free(def);
 		lilv_node_free(min);
@@ -601,11 +620,11 @@ lilv_plugin_get_supported_features(const LilvPlugin* p)
 	LilvNodes* result   = lilv_nodes_new();
 
 	LILV_FOREACH(nodes, i, optional)
-		lilv_array_append(
-			result, lilv_node_duplicate(lilv_nodes_get(optional, i)));
+		zix_tree_insert(
+			result, lilv_node_duplicate(lilv_nodes_get(optional, i)), NULL);
 	LILV_FOREACH(nodes, i, required)
-		lilv_array_append(
-			result, lilv_node_duplicate(lilv_nodes_get(required, i)));
+		zix_tree_insert(
+			result, lilv_node_duplicate(lilv_nodes_get(required, i)), NULL);
 
 	lilv_nodes_free(optional);
 	lilv_nodes_free(required);
@@ -762,7 +781,7 @@ lilv_plugin_get_uis(const LilvPlugin* p)
 			type,
 			binary);
 
-		lilv_sequence_insert(result, lilv_ui);
+		zix_tree_insert(result, lilv_ui, NULL);
 	}
 	lilv_match_end(uis);
 
@@ -776,3 +795,121 @@ lilv_plugin_get_uis(const LilvPlugin* p)
 		return NULL;
 	}
 }
+
+static size_t
+file_sink(const void* buf, size_t len, void* stream)
+{
+	FILE* file = (FILE*)stream;
+	return fwrite(buf, 1, len, file);
+}
+
+static SerdEnv*
+new_lv2_env(const SerdNode* base)
+{
+	SerdEnv* env = serd_env_new(base);
+
+#define USTR(s) ((const uint8_t*)s)
+	serd_env_set_prefix_from_strings(env, USTR("doap"), USTR(LILV_NS_DOAP));
+	serd_env_set_prefix_from_strings(env, USTR("foaf"), USTR(LILV_NS_FOAF));
+	serd_env_set_prefix_from_strings(env, USTR("lv2"),  USTR(LILV_NS_LV2));
+	serd_env_set_prefix_from_strings(env, USTR("owl"),  USTR(LILV_NS_OWL));
+	serd_env_set_prefix_from_strings(env, USTR("rdf"),  USTR(LILV_NS_RDF));
+	serd_env_set_prefix_from_strings(env, USTR("rdfs"), USTR(LILV_NS_RDFS));
+	serd_env_set_prefix_from_strings(env, USTR("xsd"),  USTR(LILV_NS_XSD));
+
+	return env;
+}
+
+static void
+maybe_write_prefixes(SerdWriter* writer, SerdEnv* env, FILE* file)
+{
+	fseek(file, 0, SEEK_END);
+	if (ftell(file) == 0) {
+		serd_env_foreach(
+			env, (SerdPrefixSink)serd_writer_set_prefix, writer);
+	} else {
+		fprintf(file, "\n");
+	}
+}
+
+LILV_API
+void
+lilv_plugin_write_description(LilvWorld*        world,
+                              const LilvPlugin* plugin,
+                              const LilvNode*   base_uri,
+                              FILE*             plugin_file)
+{
+	const LilvNode* subject   = lilv_plugin_get_uri(plugin);
+	const uint32_t  num_ports = lilv_plugin_get_num_ports(plugin);
+	const SerdNode* base      = sord_node_to_serd_node(base_uri->val.uri_val);
+	SerdEnv*        env       = new_lv2_env(base);
+
+	SerdWriter* writer = serd_writer_new(
+		SERD_TURTLE,
+		SERD_STYLE_ABBREVIATED|SERD_STYLE_CURIED,
+		env,
+		NULL,
+		file_sink,
+		plugin_file);
+
+	// Write prefixes if this is a new file
+	maybe_write_prefixes(writer, env, plugin_file);
+
+	// Write plugin description
+	SordIter* iter = lilv_world_query_internal(
+		world, subject->val.uri_val, NULL, NULL);
+	sord_write_iter(iter, writer);
+
+	// Write port descriptions
+	for (uint32_t i = 0; i < num_ports; ++i) {
+		const LilvPort* port = plugin->ports[i];
+		SordIter* iter = lilv_world_query_internal(
+			world, port->node, NULL, NULL);
+		sord_write_iter(iter, writer);
+	}
+
+	serd_writer_free(writer);
+	serd_env_free(env);
+}
+
+LILV_API
+void
+lilv_plugin_write_manifest_entry(LilvWorld*        world,
+                                 const LilvPlugin* plugin,
+                                 const LilvNode*   base_uri,
+                                 FILE*             manifest_file,
+                                 const char*       plugin_file_path)
+{
+	const LilvNode* subject = lilv_plugin_get_uri(plugin);
+	const SerdNode* base    = sord_node_to_serd_node(base_uri->val.uri_val);
+	SerdEnv*        env     = new_lv2_env(base);
+
+	SerdWriter* writer = serd_writer_new(
+		SERD_TURTLE,
+		SERD_STYLE_ABBREVIATED|SERD_STYLE_CURIED,
+		env,
+		NULL,
+		file_sink,
+		manifest_file);
+
+	// Write prefixes if this is a new file
+	maybe_write_prefixes(writer, env, manifest_file);
+
+	// Write manifest entry
+	serd_writer_write_statement(
+		writer, 0, NULL,
+		sord_node_to_serd_node(subject->val.uri_val),
+		sord_node_to_serd_node(plugin->world->rdf_a_node),
+		sord_node_to_serd_node(plugin->world->lv2_plugin_node), 0, 0);
+
+	const SerdNode file_node = serd_node_from_string(
+		SERD_URI, (const uint8_t*)plugin_file_path);
+	serd_writer_write_statement(
+		writer, 0, NULL,
+		sord_node_to_serd_node(subject->val.uri_val),
+		sord_node_to_serd_node(plugin->world->rdfs_seealso_node),
+		&file_node, 0, 0);
+
+	serd_writer_free(writer);
+	serd_env_free(env);
+}
