@@ -45,7 +45,7 @@ lilv_ui_new(LilvWorld* world,
 	free(bundle);
 
 	ui->classes = lilv_nodes_new();
-	lilv_array_append(ui->classes, type_uri);
+	zix_tree_insert(ui->classes, type_uri, NULL);
 
 	return ui;
 }
@@ -90,7 +90,7 @@ lilv_ui_is_supported(const LilvUI*       ui,
 		                                       lilv_node_as_uri(type));
 		if (q) {
 			if (ui_type) {
-				*ui_type = lilv_node_duplicate(type);
+				*ui_type = type;
 			}
 			return q;
 		}
