@@ -135,24 +135,24 @@ lilv_world_free(LilvWorld* world)
 	lilv_node_free(world->lv2_optionalFeature_val);
 	lilv_node_free(world->lv2_requiredFeature_val);
 
-	for (GSList* l = world->specs; l; l = l->next) {
-		LilvSpec* spec = (LilvSpec*)l->data;
+	for (LilvSpec* spec = world->specs; spec;) {
+		LilvSpec* next = spec->next;
 		sord_node_free(world->world, spec->spec);
 		sord_node_free(world->world, spec->bundle);
 		lilv_nodes_free(spec->data_uris);
 		free(spec);
+		spec = next;
 	}
-	g_slist_free(world->specs);
 	world->specs = NULL;
 
 	LILV_FOREACH(plugins, i, world->plugins) {
 		const LilvPlugin* p = lilv_plugins_get(world->plugins, i);
 		lilv_plugin_free((LilvPlugin*)p);
 	}
-	g_sequence_free(world->plugins);
+	zix_tree_free(world->plugins);
 	world->plugins = NULL;
 
-	g_sequence_free(world->plugin_classes);
+	zix_tree_free(world->plugin_classes);
 	world->plugin_classes = NULL;
 
 	sord_free(world->model);
@@ -278,32 +278,18 @@ lilv_header_compare_by_uri(const void* a, const void* b, void* user_data)
 	              lilv_node_as_uri(header_b->uri));
 }
 
-/** Get an element of a sequence of any object with an LilvHeader by URI. */
+/** Get an element of a collection of any object with an LilvHeader by URI. */
 struct LilvHeader*
-lilv_sequence_get_by_uri(const GSequence* const_seq,
-                         const LilvNode* uri)
+lilv_collection_get_by_uri(const ZixTree*  const_seq,
+                           const LilvNode* uri)
 {
-	GSequence*        seq = (GSequence*)const_seq;
+	ZixTree*          seq = (ZixTree*)const_seq;
 	struct LilvHeader key = { NULL, (LilvNode*)uri };
-	GSequenceIter*    i   = g_sequence_search(
-		seq, &key, lilv_header_compare_by_uri, NULL);
 
-	// i points to where plugin would be inserted (not necessarily a match)
-
-	if (!g_sequence_iter_is_end(i)) {
-		LilvPlugin* p = g_sequence_get(i);
-		if (lilv_node_equals(lilv_plugin_get_uri(p), uri)) {
-			return (struct LilvHeader*)p;
-		}
-	}
-
-	if (!g_sequence_iter_is_begin(i)) {
-		// Check if i is just past a match
-		i = g_sequence_iter_prev(i);
-		LilvPlugin* p = g_sequence_get(i);
-		if (lilv_node_equals(lilv_plugin_get_uri(p), uri)) {
-			return (struct LilvHeader*)p;
-		}
+	ZixTreeIter* i  = NULL;
+	ZixStatus    st = zix_tree_find(seq, &key, &i);
+	if (!st) {
+		return (struct LilvHeader*)zix_tree_get(i);
 	}
 
 	return NULL;
@@ -328,13 +314,15 @@ lilv_world_add_spec(LilvWorld*      world,
 		NULL);
 	FOREACH_MATCH(files) {
 		const SordNode* file_node = lilv_match_object(files);
-		lilv_array_append(spec->data_uris,
-		                  lilv_node_new_from_node(world, file_node));
+		zix_tree_insert(spec->data_uris,
+		                lilv_node_new_from_node(world, file_node),
+		                NULL);
 	}
 	lilv_match_end(files);
 
-	// Add specification to world specification sequence
-	world->specs = g_slist_prepend(world->specs, spec);
+	// Add specification to world specification list
+	spec->next   = world->specs;
+	world->specs = spec;
 }
 
 static void
@@ -344,7 +332,7 @@ lilv_world_add_plugin(LilvWorld*      world,
                       const SordNode* dyn_manifest_lib,
                       const SordNode* bundle_node)
 {
-	LilvNode* plugin_uri  = lilv_node_new_from_node(world, plugin_node);
+	LilvNode* plugin_uri = lilv_node_new_from_node(world, plugin_node);
 
 	const LilvPlugin* last = lilv_plugins_get_by_uri(world->plugins, plugin_uri);
 	if (last) {
@@ -361,8 +349,9 @@ lilv_world_add_plugin(LilvWorld*      world,
 	LilvPlugin* plugin     = lilv_plugin_new(world, plugin_uri, bundle_uri);
 
 	// Add manifest as plugin data file (as if it were rdfs:seeAlso)
-	lilv_array_append(plugin->data_uris,
-	                  lilv_new_uri(world, (const char*)manifest_uri->buf));
+	zix_tree_insert(plugin->data_uris,
+	                lilv_new_uri(world, (const char*)manifest_uri->buf),
+	                NULL);
 
 	// Set dynamic manifest library URI, if applicable
 	if (dyn_manifest_lib) {
@@ -378,13 +367,14 @@ lilv_world_add_plugin(LilvWorld*      world,
 		NULL);
 	FOREACH_MATCH(files) {
 		const SordNode* file_node = lilv_match_object(files);
-		lilv_array_append(plugin->data_uris,
-		                  lilv_node_new_from_node(world, file_node));
+		zix_tree_insert(plugin->data_uris,
+		                lilv_node_new_from_node(world, file_node),
+		                NULL);
 	}
 	lilv_match_end(files);
 
 	// Add plugin to world plugin sequence
-	lilv_sequence_insert(world->plugins, plugin);
+	zix_tree_insert(world->plugins, plugin, NULL);
 }
 
 static void
@@ -430,6 +420,7 @@ lilv_world_load_dyn_manifest(LilvWorld* world,
 		const char*     lib_path = lilv_uri_to_path((const char*)lib_uri);
 		if (!lib_path) {
 			LILV_ERROR("No dynamic manifest library path\n");
+			lilv_match_end(binaries);
 			continue;
 		}
 
@@ -437,6 +428,7 @@ lilv_world_load_dyn_manifest(LilvWorld* world,
 		void* lib = dlopen(lib_path, RTLD_LAZY);
 		if (!lib) {
 			LILV_ERRORF("Failed to open dynamic manifest library `%s'\n", lib_path);
+			lilv_match_end(binaries);
 			continue;
 		}
 
@@ -445,6 +437,7 @@ lilv_world_load_dyn_manifest(LilvWorld* world,
 		OpenFunc open_func = (OpenFunc)lilv_dlfunc(lib, "lv2_dyn_manifest_open");
 		if (!open_func || open_func(&handle, &dman_features)) {
 			LILV_ERRORF("Failed to find `lv2_dyn_manifest_open' in `%s'\n", lib_path);
+			lilv_match_end(binaries);
 			dlclose(lib);
 			continue;
 		}
@@ -456,6 +449,7 @@ lilv_world_load_dyn_manifest(LilvWorld* world,
 		if (!get_subjects_func) {
 			LILV_ERRORF("Failed to find `lv2_dyn_manifest_get_subjects' in `%s'\n",
 			            lib_path);
+			lilv_match_end(binaries);
 			dlclose(lib);
 			continue;
 		}
@@ -466,7 +460,7 @@ lilv_world_load_dyn_manifest(LilvWorld* world,
 		rewind(fd);
 
 		// Parse generated data file
-		SerdEnv*    env    = serd_env_new(bundle_node);
+		SerdEnv*    env    = serd_env_new(sord_node_to_serd_node(bundle_node));
 		SerdReader* reader = sord_new_reader(
 			world->model, env, SERD_TURTLE, bundle_node);
 		serd_reader_read_file_handle(reader, fd,
@@ -491,6 +485,7 @@ lilv_world_load_dyn_manifest(LilvWorld* world,
 		}
 		lilv_match_end(plug_results);
 
+		lilv_match_end(binaries);
 		dlclose(lib);
 	}
 	lilv_match_end(dmanifests);
@@ -676,8 +671,7 @@ lilv_world_load_path(LilvWorld*  world,
 static void
 lilv_world_load_specifications(LilvWorld* world)
 {
-	for (GSList* l = world->specs; l; l = l->next) {
-		LilvSpec* spec = (LilvSpec*)l->data;
+	for (LilvSpec* spec = world->specs; spec; spec = spec->next) {
 		LILV_FOREACH(nodes, f, spec->data_uris) {
 			LilvNode*      file     = lilv_collection_get(spec->data_uris, f);
 			const uint8_t* file_uri = (const uint8_t*)lilv_node_as_uri(file);
@@ -685,7 +679,7 @@ lilv_world_load_specifications(LilvWorld* world)
 				sord_node_to_serd_node(file->val.uri_val));
 			SerdReader*    reader   = sord_new_reader(world->model, env,
 			                                          SERD_TURTLE, NULL);
-			serd_reader_add_blank_prefix(reader, 
+			serd_reader_add_blank_prefix(reader,
 			                             lilv_world_blank_node_prefix(world));
 			serd_reader_read_file(reader, file_uri);
 			serd_reader_free(reader);
@@ -746,8 +740,8 @@ lilv_world_load_plugin_classes(LilvWorld* world)
 			continue;
 		}
 
-		const SordNode*       label_node = lilv_match_object(labels);
-		const uint8_t* label      = (const uint8_t*)sord_node_get_string(label_node);
+		const SordNode* label_node = lilv_match_object(labels);
+		const uint8_t*  label      = sord_node_get_string(label_node);
 		lilv_match_end(labels);
 
 		LilvPluginClasses* classes = world->plugin_classes;
@@ -755,7 +749,7 @@ lilv_world_load_plugin_classes(LilvWorld* world)
 			world, parent_node, class_node, (const char*)label);
 
 		if (pclass) {
-			lilv_sequence_insert(classes, pclass);
+			zix_tree_insert(classes, pclass, NULL);
 		}
 	}
 	lilv_match_end(classes);
